@@ -41,8 +41,8 @@
 #include <deque>
 #include <map>
 #include <vector>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "a2netcompat.h"
 #include "TimerA2.h"
 #include "a2io.h"
@@ -80,15 +80,15 @@ private:
 
   std::string sessionId_;
 
-  SharedHandle<EventPoll> eventPoll_;
+  std::unique_ptr<EventPoll> eventPoll_;
 
-  SharedHandle<StatCalc> statCalc_;
+  std::unique_ptr<StatCalc> statCalc_;
 
   int haltRequested_;
 
   class SocketPoolEntry {
   private:
-    SharedHandle<SocketCore> socket_;
+    std::shared_ptr<SocketCore> socket_;
     // protocol specific option string
     std::string options_;
 
@@ -96,18 +96,18 @@ private:
 
     Timer registeredTime_;
   public:
-    SocketPoolEntry(const SharedHandle<SocketCore>& socket,
+    SocketPoolEntry(const std::shared_ptr<SocketCore>& socket,
                     const std::string& option,
                     time_t timeout);
 
-    SocketPoolEntry(const SharedHandle<SocketCore>& socket,
+    SocketPoolEntry(const std::shared_ptr<SocketCore>& socket,
                     time_t timeout);
 
     ~SocketPoolEntry();
 
     bool isTimeout() const;
 
-    const SharedHandle<SocketCore>& getSocket() const
+    const std::shared_ptr<SocketCore>& getSocket() const
     {
       return socket_;
     }
@@ -131,12 +131,12 @@ private:
   int64_t refreshInterval_;
   Timer lastRefresh_;
 
-  std::deque<Command*> routineCommands_;
+  std::deque<std::unique_ptr<Command>> routineCommands_;
 
-  SharedHandle<CookieStorage> cookieStorage_;
+  std::unique_ptr<CookieStorage> cookieStorage_;
 
 #ifdef ENABLE_BITTORRENT
-  SharedHandle<BtRegistry> btRegistry_;
+  std::unique_ptr<BtRegistry> btRegistry_;
 #endif // ENABLE_BITTORRENT
 
   CUIDCounter cuidCounter_;
@@ -145,12 +145,12 @@ private:
   ares_addr_node* asyncDNSServers_;
 #endif // HAVE_ARES_ADDR_NODE
 
-  SharedHandle<DNSCache> dnsCache_;
+  std::unique_ptr<DNSCache> dnsCache_;
 
-  SharedHandle<AuthConfigFactory> authConfigFactory_;
+  std::unique_ptr<AuthConfigFactory> authConfigFactory_;
 
 #ifdef ENABLE_WEBSOCKET
-  SharedHandle<rpc::WebSocketSessionMan> webSocketSessionMan_;
+  std::unique_ptr<rpc::WebSocketSessionMan> webSocketSessionMan_;
 #endif // ENABLE_WEBSOCKET
 
   /**
@@ -167,13 +167,13 @@ private:
   std::multimap<std::string, SocketPoolEntry>::iterator
   findSocketPoolEntry(const std::string& key);
 
-  std::deque<Command*> commands_;
-  SharedHandle<RequestGroupMan> requestGroupMan_;
-  SharedHandle<FileAllocationMan> fileAllocationMan_;
-  SharedHandle<CheckIntegrityMan> checkIntegrityMan_;
+  std::deque<std::unique_ptr<Command>> commands_;
+  std::unique_ptr<RequestGroupMan> requestGroupMan_;
+  std::unique_ptr<FileAllocationMan> fileAllocationMan_;
+  std::unique_ptr<CheckIntegrityMan> checkIntegrityMan_;
   Option* option_;
 public:
-  DownloadEngine(const SharedHandle<EventPoll>& eventPoll);
+  DownloadEngine(std::unique_ptr<EventPoll> eventPoll);
 
   ~DownloadEngine();
 
@@ -183,47 +183,47 @@ public:
   // processed. Otherwise, returns 0.
   int run(bool oneshot=false);
 
-  bool addSocketForReadCheck(const SharedHandle<SocketCore>& socket,
+  bool addSocketForReadCheck(const std::shared_ptr<SocketCore>& socket,
                              Command* command);
-  bool deleteSocketForReadCheck(const SharedHandle<SocketCore>& socket,
+  bool deleteSocketForReadCheck(const std::shared_ptr<SocketCore>& socket,
                                 Command* command);
-  bool addSocketForWriteCheck(const SharedHandle<SocketCore>& socket,
+  bool addSocketForWriteCheck(const std::shared_ptr<SocketCore>& socket,
                               Command* command);
-  bool deleteSocketForWriteCheck(const SharedHandle<SocketCore>& socket,
+  bool deleteSocketForWriteCheck(const std::shared_ptr<SocketCore>& socket,
                                  Command* command);
 
 #ifdef ENABLE_ASYNC_DNS
 
-  bool addNameResolverCheck(const SharedHandle<AsyncNameResolver>& resolver,
+  bool addNameResolverCheck(const std::shared_ptr<AsyncNameResolver>& resolver,
                             Command* command);
-  bool deleteNameResolverCheck(const SharedHandle<AsyncNameResolver>& resolver,
+  bool deleteNameResolverCheck(const std::shared_ptr<AsyncNameResolver>& resolver,
                                Command* command);
 #endif // ENABLE_ASYNC_DNS
 
-  void addCommand(const std::vector<Command*>& commands);
+  void addCommand(std::vector<std::unique_ptr<Command>> commands);
 
-  void addCommand(Command* command);
+  void addCommand(std::unique_ptr<Command> command);
 
-  const SharedHandle<RequestGroupMan>& getRequestGroupMan() const
+  const std::unique_ptr<RequestGroupMan>& getRequestGroupMan() const
   {
     return requestGroupMan_;
   }
 
-  void setRequestGroupMan(const SharedHandle<RequestGroupMan>& rgman);
+  void setRequestGroupMan(std::unique_ptr<RequestGroupMan> rgman);
 
-  const SharedHandle<FileAllocationMan>& getFileAllocationMan() const
+  const std::unique_ptr<FileAllocationMan>& getFileAllocationMan() const
   {
     return fileAllocationMan_;
   }
 
-  void setFileAllocationMan(const SharedHandle<FileAllocationMan>& faman);
+  void setFileAllocationMan(std::unique_ptr<FileAllocationMan> faman);
 
-  const SharedHandle<CheckIntegrityMan>& getCheckIntegrityMan() const
+  const std::unique_ptr<CheckIntegrityMan>& getCheckIntegrityMan() const
   {
     return checkIntegrityMan_;
   }
 
-  void setCheckIntegrityMan(const SharedHandle<CheckIntegrityMan>& ciman);
+  void setCheckIntegrityMan(std::unique_ptr<CheckIntegrityMan> ciman);
 
   Option* getOption() const
   {
@@ -235,7 +235,7 @@ public:
     option_ = op;
   }
 
-  void setStatCalc(const SharedHandle<StatCalc>& statCalc);
+  void setStatCalc(std::unique_ptr<StatCalc> statCalc);
 
   bool isHaltRequested() const
   {
@@ -253,62 +253,59 @@ public:
 
   void setNoWait(bool b);
 
-  void addRoutineCommand(Command* command);
+  void addRoutineCommand(std::unique_ptr<Command> command);
 
   void poolSocket(const std::string& ipaddr, uint16_t port,
                   const std::string& username,
                   const std::string& proxyhost, uint16_t proxyport,
-                  const SharedHandle<SocketCore>& sock,
+                  const std::shared_ptr<SocketCore>& sock,
                   const std::string& options,
                   time_t timeout = 15);
 
-  void poolSocket(const SharedHandle<Request>& request,
+  void poolSocket(const std::shared_ptr<Request>& request,
                   const std::string& username,
-                  const SharedHandle<Request>& proxyRequest,
-                  const SharedHandle<SocketCore>& socket,
+                  const std::shared_ptr<Request>& proxyRequest,
+                  const std::shared_ptr<SocketCore>& socket,
                   const std::string& options,
                   time_t timeout = 15);
 
   void poolSocket(const std::string& ipaddr, uint16_t port,
                   const std::string& proxyhost, uint16_t proxyport,
-                  const SharedHandle<SocketCore>& sock,
+                  const std::shared_ptr<SocketCore>& sock,
                   time_t timeout = 15);
 
-  void poolSocket(const SharedHandle<Request>& request,
-                  const SharedHandle<Request>& proxyRequest,
-                  const SharedHandle<SocketCore>& socket,
+  void poolSocket(const std::shared_ptr<Request>& request,
+                  const std::shared_ptr<Request>& proxyRequest,
+                  const std::shared_ptr<SocketCore>& socket,
                   time_t timeout = 15);
 
-  SharedHandle<SocketCore> popPooledSocket
+  std::shared_ptr<SocketCore> popPooledSocket
   (const std::string& ipaddr,
    uint16_t port,
    const std::string& proxyhost, uint16_t proxyport);
 
-  SharedHandle<SocketCore> popPooledSocket
+  std::shared_ptr<SocketCore> popPooledSocket
   (std::string& options,
    const std::string& ipaddr,
    uint16_t port,
    const std::string& username,
    const std::string& proxyhost, uint16_t proxyport);
 
-  SharedHandle<SocketCore>
+  std::shared_ptr<SocketCore>
   popPooledSocket
   (const std::vector<std::string>& ipaddrs, uint16_t port);
 
-  SharedHandle<SocketCore>
+  std::shared_ptr<SocketCore>
   popPooledSocket
   (std::string& options,
    const std::vector<std::string>& ipaddrs,
    uint16_t port,
    const std::string& username);
 
-  const SharedHandle<CookieStorage>& getCookieStorage() const
-  {
-    return cookieStorage_;
-  }
+  const std::unique_ptr<CookieStorage>& getCookieStorage() const;
 
 #ifdef ENABLE_BITTORRENT
-  const SharedHandle<BtRegistry>& getBtRegistry() const
+  const std::unique_ptr<BtRegistry>& getBtRegistry() const
   {
     return btRegistry_;
   }
@@ -334,12 +331,9 @@ public:
 
   void removeCachedIPAddress(const std::string& hostname, uint16_t port);
 
-  void setAuthConfigFactory(const SharedHandle<AuthConfigFactory>& factory);
+  void setAuthConfigFactory(std::unique_ptr<AuthConfigFactory> factory);
 
-  const SharedHandle<AuthConfigFactory>& getAuthConfigFactory() const
-  {
-    return authConfigFactory_;
-  }
+  const std::unique_ptr<AuthConfigFactory>& getAuthConfigFactory() const;
 
   void setRefreshInterval(int64_t interval);
 
@@ -358,9 +352,9 @@ public:
 #endif // HAVE_ARES_ADDR_NODE
 
 #ifdef ENABLE_WEBSOCKET
-  void setWebSocketSessionMan
-  (const SharedHandle<rpc::WebSocketSessionMan>& wsman);
-  const SharedHandle<rpc::WebSocketSessionMan>& getWebSocketSessionMan() const
+  void setWebSocketSessionMan(std::unique_ptr<rpc::WebSocketSessionMan> wsman);
+  const std::unique_ptr<rpc::WebSocketSessionMan>& getWebSocketSessionMan()
+    const
   {
     return webSocketSessionMan_;
   }
