@@ -40,15 +40,15 @@
 #include <cassert>
 #include <string>
 #include <vector>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "FileEntry.h"
 
 namespace aria2 {
 
 class Request;
 class Segment;
-class Range;
+struct Range;
 class Option;
 class CookieStorage;
 class AuthConfigFactory;
@@ -56,14 +56,14 @@ class AuthConfig;
 
 class HttpRequest {
 private:
-  
+
   static const std::string USER_AGENT;
 
-  SharedHandle<Request> request_;
+  std::shared_ptr<Request> request_;
 
-  SharedHandle<FileEntry> fileEntry_;
+  std::shared_ptr<FileEntry> fileEntry_;
 
-  SharedHandle<Segment> segment_;
+  std::shared_ptr<Segment> segment_;
 
   bool contentEncodingEnabled_;
 
@@ -71,22 +71,30 @@ private:
 
   std::vector<std::string> headers_;
 
-  std::vector<std::string> acceptTypes_;
+  // If true, metalink content types are sent in Accept header field.
+  bool acceptMetalink_;
 
-  SharedHandle<CookieStorage> cookieStorage_;
+  CookieStorage* cookieStorage_;
 
-  SharedHandle<AuthConfigFactory> authConfigFactory_;
+  AuthConfigFactory* authConfigFactory_;
 
   const Option* option_;
 
-  SharedHandle<AuthConfig> authConfig_;
+  std::unique_ptr<AuthConfig> authConfig_;
 
-  SharedHandle<Request> proxyRequest_;
+  std::shared_ptr<Request> proxyRequest_;
 
   bool noCache_;
 
   bool acceptGzip_;
 
+  // Historically, aria2 did not specify end byte marker unless http
+  // pipelining is enabled. Sometimes end byte is known because the
+  // segment/piece ahead of this request was already acquired. In this
+  // case, specifying end byte enables to reuse connection. To achieve
+  // this, if endOffsetOverride_ is more than 0, its value - 1 is used
+  // as an end byte. Please note that FTP protocol cannot specify end
+  // bytes and it is also true if it is used via HTTP proxy.
   int64_t endOffsetOverride_;
 
   std::string ifModSinceHeader_;
@@ -96,14 +104,14 @@ public:
   HttpRequest();
   ~HttpRequest();
 
-  const SharedHandle<Segment>& getSegment() const
+  const std::shared_ptr<Segment>& getSegment() const
   {
     return segment_;
   }
 
-  void setSegment(const SharedHandle<Segment>& segment);
+  void setSegment(const std::shared_ptr<Segment>& segment);
 
-  void setRequest(const SharedHandle<Request>& request);
+  void setRequest(const std::shared_ptr<Request>& request);
 
   int64_t getEntityLength() const;
 
@@ -116,7 +124,7 @@ public:
   const std::string& getProtocol() const;
 
   const std::string& getCurrentURI() const;
-  
+
   const std::string& getDir() const;
 
   const std::string& getFile() const;
@@ -127,15 +135,15 @@ public:
 
   std::string getURIHost() const;
 
-  SharedHandle<Range> getRange() const;
+  Range getRange() const;
 
   /**
    * Inspects whether the specified response range is satisfiable
    * with request range.
    */
-  bool isRangeSatisfied(const SharedHandle<Range>& range) const;
+  bool isRangeSatisfied(const Range& range) const;
 
-  const SharedHandle<Request>& getRequest() const
+  const std::shared_ptr<Request>& getRequest() const
   {
     return request_;
   }
@@ -164,7 +172,7 @@ public:
   void disableContentEncoding();
 
   void setUserAgent(const std::string& userAgent);
-  
+
   // accepts multiline headers, delimited by LF
   void addHeader(const std::string& headers);
 
@@ -172,28 +180,24 @@ public:
 
   void addAcceptType(const std::string& type);
 
-  template<typename InputIterator>
-  void addAcceptType(InputIterator first, InputIterator last)
+  void setAcceptMetalink(bool f)
   {
-    acceptTypes_.insert(acceptTypes_.end(), first, last);
+    acceptMetalink_ = f;
   }
 
-  void setCookieStorage(const SharedHandle<CookieStorage>& cookieStorage);
+  void setCookieStorage(CookieStorage* cookieStorage);
 
-  const SharedHandle<CookieStorage>& getCookieStorage() const
-  {
-    return cookieStorage_;
-  }
+  CookieStorage* getCookieStorage() const;
 
-  void setAuthConfigFactory
-  (const SharedHandle<AuthConfigFactory>& factory, const Option* option);
+  void setAuthConfigFactory(AuthConfigFactory* factory);
+  void setOption(const Option* option);
 
   /*
    * To use proxy, pass proxy string to Request::setUri() and set it this
    * object.
    */
-  void setProxyRequest(const SharedHandle<Request>& proxyRequest);
-  
+  void setProxyRequest(const std::shared_ptr<Request>& proxyRequest);
+
   /*
    * Returns true if non-Null proxy request is set by setProxyRequest().
    * Otherwise, returns false.
@@ -206,11 +210,11 @@ public:
 
   // Returns AuthConfig used in the last invocation of
   // createRequest().
-  const SharedHandle<AuthConfig>& getAuthConfig() const;
+  const std::unique_ptr<AuthConfig>& getAuthConfig() const;
 
-  void setFileEntry(const SharedHandle<FileEntry>& fileEntry);
+  void setFileEntry(const std::shared_ptr<FileEntry>& fileEntry);
 
-  const SharedHandle<FileEntry>& getFileEntry() const
+  const std::shared_ptr<FileEntry>& getFileEntry() const
   {
     return fileEntry_;
   }
