@@ -68,11 +68,9 @@
 namespace aria2 {
 
 HttpResponse::HttpResponse()
-  : cuid_(0)
+  : cuid_{0}
 {}
 
-HttpResponse::~HttpResponse() {}
-
 void HttpResponse::validateResponse() const
 {
   int statusCode = getStatusCode();
@@ -96,16 +94,16 @@ void HttpResponse::validateResponse() const
   } else if(statusCode == 200 || statusCode == 206) {
     if(!httpHeader_->defined(HttpHeader::TRANSFER_ENCODING)) {
       // compare the received range against the requested range
-      RangeHandle responseRange = httpHeader_->getRange();
+      Range responseRange = httpHeader_->getRange();
       if(!httpRequest_->isRangeSatisfied(responseRange)) {
         throw DL_ABORT_EX2
           (fmt(EX_INVALID_RANGE_HEADER,
-               static_cast<long long int>(httpRequest_->getStartByte()),
-               static_cast<long long int>(httpRequest_->getEndByte()),
-               static_cast<long long int>(httpRequest_->getEntityLength()),
-               static_cast<long long int>(responseRange->getStartByte()),
-               static_cast<long long int>(responseRange->getEndByte()),
-               static_cast<long long int>(responseRange->getEntityLength())),
+               httpRequest_->getStartByte(),
+               httpRequest_->getEndByte(),
+               httpRequest_->getEntityLength(),
+               responseRange.startByte,
+               responseRange.endByte,
+               responseRange.entityLength),
            error_code::CANNOT_RESUME);
       }
     }
@@ -126,22 +124,20 @@ std::string HttpResponse::determinFilename() const
                           httpRequest_->getFile().end());
     if(file.empty()) {
       return "index.html";
-    } else {
-      return file;
     }
-  } else {
-    A2_LOG_INFO(fmt(MSG_CONTENT_DISPOSITION_DETECTED,
-                    cuid_,
-                    contentDisposition.c_str()));
-    return contentDisposition;
+    return file;
   }
+  A2_LOG_INFO(fmt(MSG_CONTENT_DISPOSITION_DETECTED,
+                  cuid_,
+                  contentDisposition.c_str()));
+  return contentDisposition;
 }
 
 void HttpResponse::retrieveCookie()
 {
   Time now;
-  std::pair<std::multimap<std::string, std::string>::const_iterator,
-            std::multimap<std::string, std::string>::const_iterator> r =
+  std::pair<std::multimap<int, std::string>::const_iterator,
+            std::multimap<int, std::string>::const_iterator> r =
     httpHeader_->equalRange(HttpHeader::SET_COOKIE);
   for(; r.first != r.second; ++r.first) {
     httpRequest_->getCookieStorage()->parseAndStore
@@ -162,7 +158,6 @@ bool HttpResponse::isRedirect() const
 
 void HttpResponse::processRedirect()
 {
-  
   if(httpRequest_->getRequest()->redirectUri
      (util::percentEncodeMini(getRedirectURI()))) {
     A2_LOG_INFO(fmt(MSG_REDIRECT,
@@ -170,7 +165,7 @@ void HttpResponse::processRedirect()
                     httpRequest_->getRequest()->getCurrentUri().c_str()));
   } else {
     throw DL_RETRY_EX
-      (fmt("CUID#%lld - Redirect to %s failed. It may not be a valid URI.",
+      (fmt("CUID#%" PRId64 " - Redirect to %s failed. It may not be a valid URI.",
            cuid_,
            httpRequest_->getRequest()->getCurrentUri().c_str()));
   }
@@ -192,17 +187,17 @@ const std::string& HttpResponse::getTransferEncoding() const
   return httpHeader_->find(HttpHeader::TRANSFER_ENCODING);
 }
 
-SharedHandle<StreamFilter> HttpResponse::getTransferEncodingStreamFilter() const
+std::unique_ptr<StreamFilter>
+HttpResponse::getTransferEncodingStreamFilter() const
 {
-  SharedHandle<StreamFilter> filter;
   // TODO Transfer-Encoding header field can contains multiple tokens. We should
   // parse the field and retrieve each token.
   if(isTransferEncodingSpecified()) {
-    if(getTransferEncoding() == HttpHeader::CHUNKED) {
-      filter.reset(new ChunkedDecodingStreamFilter());
+    if(util::strieq(getTransferEncoding(), "chunked")) {
+      return make_unique<ChunkedDecodingStreamFilter>();
     }
   }
-  return filter;
+  return nullptr;
 }
 
 bool HttpResponse::isContentEncodingSpecified() const
@@ -215,33 +210,33 @@ const std::string& HttpResponse::getContentEncoding() const
   return httpHeader_->find(HttpHeader::CONTENT_ENCODING);
 }
 
-SharedHandle<StreamFilter> HttpResponse::getContentEncodingStreamFilter() const
+std::unique_ptr<StreamFilter>
+HttpResponse::getContentEncodingStreamFilter() const
 {
-  SharedHandle<StreamFilter> filter;
 #ifdef HAVE_ZLIB
-  if(getContentEncoding() == HttpHeader::GZIP ||
-     getContentEncoding() == HttpHeader::DEFLATE) {
-    filter.reset(new GZipDecodingStreamFilter());
+  if(util::strieq(getContentEncoding(), "gzip") ||
+     util::strieq(getContentEncoding(), "deflate")) {
+    return make_unique<GZipDecodingStreamFilter>();
   }
 #endif // HAVE_ZLIB
-  return filter;
+  return nullptr;
 }
 
-off_t HttpResponse::getContentLength() const
+int64_t HttpResponse::getContentLength() const
 {
   if(!httpHeader_) {
     return 0;
   } else {
-    return httpHeader_->getRange()->getContentLength();
+    return httpHeader_->getRange().getContentLength();
   }
 }
 
-off_t HttpResponse::getEntityLength() const
+int64_t HttpResponse::getEntityLength() const
 {
   if(!httpHeader_) {
     return 0;
   } else {
-    return httpHeader_->getRange()->getEntityLength();
+    return httpHeader_->getRange().entityLength;
   }
 }
 
@@ -257,14 +252,19 @@ std::string HttpResponse::getContentType() const
   }
 }
 
-void HttpResponse::setHttpHeader(const SharedHandle<HttpHeader>& httpHeader)
+void HttpResponse::setHttpHeader(std::unique_ptr<HttpHeader> httpHeader)
 {
-  httpHeader_ = httpHeader;
+  httpHeader_ = std::move(httpHeader);
 }
 
-void HttpResponse::setHttpRequest(const SharedHandle<HttpRequest>& httpRequest)
+const std::unique_ptr<HttpHeader>& HttpResponse::getHttpHeader() const
 {
-  httpRequest_ = httpRequest;
+  return httpHeader_;
+}
+
+void HttpResponse::setHttpRequest(std::unique_ptr<HttpRequest> httpRequest)
+{
+  httpRequest_ = std::move(httpRequest);
 }
 
 int HttpResponse::getStatusCode() const
@@ -272,16 +272,6 @@ int HttpResponse::getStatusCode() const
   return httpHeader_->getStatusCode();
 }
 
-bool HttpResponse::hasRetryAfter() const
-{
-  return httpHeader_->defined(HttpHeader::RETRY_AFTER);
-}
-
-time_t HttpResponse::getRetryAfter() const
-{
-  return httpHeader_->findAsInt(HttpHeader::RETRY_AFTER);
-}
-
 Time HttpResponse::getLastModifiedTime() const
 {
   return Time::parseHTTPDate(httpHeader_->find(HttpHeader::LAST_MODIFIED));
@@ -289,25 +279,7 @@ Time HttpResponse::getLastModifiedTime() const
 
 bool HttpResponse::supportsPersistentConnection() const
 {
-  const std::string& connection = httpHeader_->find(HttpHeader::CONNECTION);
-  const std::string& version = httpHeader_->getVersion();
-  const std::string& proxyConn =
-    httpHeader_->find(HttpHeader::PROXY_CONNECTION);
-  return
-    util::strifind(connection.begin(),
-                   connection.end(),
-                   HttpHeader::CLOSE.begin(),
-                   HttpHeader::CLOSE.end()) == connection.end() &&
-    (version == HttpHeader::HTTP_1_1 ||
-     util::strifind(connection.begin(),
-                    connection.end(),
-                    HttpHeader::KEEP_ALIVE.begin(),
-                    HttpHeader::KEEP_ALIVE.end()) != connection.end()) &&
-    (!httpRequest_->isProxyRequestSet() ||
-     util::strifind(proxyConn.begin(),
-                    proxyConn.end(),
-                    HttpHeader::KEEP_ALIVE.begin(),
-                    HttpHeader::KEEP_ALIVE.end()) != proxyConn.end());
+  return httpHeader_->isKeepAlive();
 }
 
 namespace {
@@ -325,9 +297,8 @@ bool parseMetalinkHttpLink(MetalinkHttpEntry& result, const std::string& s)
             std::string::const_iterator> p = util::stripIter(first+1, last);
   if(p.first == p.second) {
     return false;
-  } else {
-    result.uri.assign(p.first, p.second);
   }
+  result.uri.assign(p.first, p.second);
   last = std::find(last, s.end(), ';');
   if(last != s.end()) {
     ++last;
@@ -373,10 +344,10 @@ bool parseMetalinkHttpLink(MetalinkHttpEntry& result, const std::string& s)
 // Link header field is defined by http://tools.ietf.org/html/rfc5988.
 void HttpResponse::getMetalinKHttpEntries
 (std::vector<MetalinkHttpEntry>& result,
- const SharedHandle<Option>& option) const
+ const std::shared_ptr<Option>& option) const
 {
-  std::pair<std::multimap<std::string, std::string>::const_iterator,
-            std::multimap<std::string, std::string>::const_iterator> p =
+  std::pair<std::multimap<int, std::string>::const_iterator,
+            std::multimap<int, std::string>::const_iterator> p =
     httpHeader_->equalRange(HttpHeader::LINK);
   for(; p.first != p.second; ++p.first) {
     MetalinkHttpEntry e;
@@ -389,15 +360,13 @@ void HttpResponse::getMetalinKHttpEntries
     if(option->defined(PREF_METALINK_LOCATION)) {
       const std::string& loc = option->get(PREF_METALINK_LOCATION);
       util::split(loc.begin(), loc.end(), std::back_inserter(locs), ',', true);
-      for(std::vector<std::string>::iterator i = locs.begin(), eoi = locs.end();
-          i != eoi; ++i) {
-        util::lowercase(*i);
+      for (auto& l: locs) {
+        util::lowercase(l);
       }
     }
-    for(std::vector<MetalinkHttpEntry>::iterator i = result.begin(),
-          eoi = result.end(); i != eoi; ++i) {
-      if(std::find(locs.begin(), locs.end(), (*i).geo) != locs.end()) {
-        (*i).pri -= 999999;
+    for (auto& r: result) {
+      if(std::find(locs.begin(), locs.end(), r.geo) != locs.end()) {
+        r.pri -= 999999;
       }
     }
   }
@@ -410,8 +379,8 @@ void HttpResponse::getMetalinKHttpEntries
 void HttpResponse::getDigest(std::vector<Checksum>& result) const
 {
   using std::swap;
-  std::pair<std::multimap<std::string, std::string>::const_iterator,
-            std::multimap<std::string, std::string>::const_iterator> p =
+  std::pair<std::multimap<int, std::string>::const_iterator,
+            std::multimap<int, std::string>::const_iterator> p =
     httpHeader_->equalRange(HttpHeader::DIGEST);
   for(; p.first != p.second; ++p.first) {
     const std::string& s = (*p.first).second;
@@ -435,10 +404,9 @@ void HttpResponse::getDigest(std::vector<Checksum>& result) const
   }
   std::sort(result.begin(), result.end(), HashTypeStronger());
   std::vector<Checksum> temp;
-  for(std::vector<Checksum>::iterator i = result.begin(),
-        eoi = result.end(); i != eoi;) {
+  for(auto i = result.begin(), eoi = result.end(); i != eoi;) {
     bool ok = true;
-    std::vector<Checksum>::iterator j = i+1;
+    auto j = i+1;
     for(; j != eoi; ++j) {
       if((*i).getHashType() != (*j).getHashType()) {
         break;
