@@ -88,145 +88,145 @@ void CookieHelperTest::testParse()
   {
     std::string str = "ID=123456789; expires=Sun, 10-Jun-2007 11:00:00 GMT;"
       "path=/foo; domain=localhost; secure;httpOnly   ";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("ID"), c.getName());
-    CPPUNIT_ASSERT_EQUAL(std::string("123456789"), c.getValue());
-    CPPUNIT_ASSERT_EQUAL((time_t)1181473200, c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
-    CPPUNIT_ASSERT_EQUAL(std::string("localhost"), c.getDomain());
-    CPPUNIT_ASSERT(!c.getHostOnly());
-    CPPUNIT_ASSERT_EQUAL(std::string("/foo"), c.getPath());
-    CPPUNIT_ASSERT(c.getSecure());
-    CPPUNIT_ASSERT(c.getHttpOnly());
-    CPPUNIT_ASSERT_EQUAL((time_t)141, c.getCreationTime());
-    CPPUNIT_ASSERT_EQUAL((time_t)141, c.getLastAccessTime());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("ID"), c->getName());
+    CPPUNIT_ASSERT_EQUAL(std::string("123456789"), c->getValue());
+    CPPUNIT_ASSERT_EQUAL((time_t)1181473200, c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
+    CPPUNIT_ASSERT_EQUAL(std::string("localhost"), c->getDomain());
+    CPPUNIT_ASSERT(!c->getHostOnly());
+    CPPUNIT_ASSERT_EQUAL(std::string("/foo"), c->getPath());
+    CPPUNIT_ASSERT(c->getSecure());
+    CPPUNIT_ASSERT(c->getHttpOnly());
+    CPPUNIT_ASSERT_EQUAL((time_t)141, c->getCreationTime());
+    CPPUNIT_ASSERT_EQUAL((time_t)141, c->getLastAccessTime());
   }
   {
     std::string str = "id=; Max-Age=0;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("id"), c.getName());
-    CPPUNIT_ASSERT_EQUAL((time_t)0, c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
-    CPPUNIT_ASSERT_EQUAL(std::string("localhost"), c.getDomain());
-    CPPUNIT_ASSERT(c.getHostOnly());
-    CPPUNIT_ASSERT_EQUAL(std::string("/"), c.getPath());
-    CPPUNIT_ASSERT(!c.getSecure());
-    CPPUNIT_ASSERT(!c.getHttpOnly());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("id"), c->getName());
+    CPPUNIT_ASSERT_EQUAL((time_t)0, c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
+    CPPUNIT_ASSERT_EQUAL(std::string("localhost"), c->getDomain());
+    CPPUNIT_ASSERT(c->getHostOnly());
+    CPPUNIT_ASSERT_EQUAL(std::string("/"), c->getPath());
+    CPPUNIT_ASSERT(!c->getSecure());
+    CPPUNIT_ASSERT(!c->getHttpOnly());
   }
   {
     std::string str = "id=; Max-Age=-100;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL((time_t)0, c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL((time_t)0, c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
   }
   {
     std::string str = "id=; Max-Age=100;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL((time_t)creationDate+100, c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL((time_t)creationDate+100, c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
   }
   {
     std::string str = "id=; Max-Age=9223372036854775807;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::numeric_limits<time_t>::max(), c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::numeric_limits<time_t>::max(),
+                         c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
   }
   {
     std::string str = "id=; Max-Age=X;";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/", creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/", creationDate));
   }
   {
     std::string str = "id=; Max-Age=100garbage;";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/", creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/", creationDate));
   }
   {
     std::string str = "id=; Max-Age=100;expires=Sun, 10-Jun-2007 11:00:00 GMT;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL((time_t)creationDate+100, c.getExpiryTime());
-    CPPUNIT_ASSERT(c.getPersistent());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL((time_t)creationDate+100, c->getExpiryTime());
+    CPPUNIT_ASSERT(c->getPersistent());
   }
   {
     // Cookie data cannot be parsed.
     std::string str = "id=; expires=2007-10-01 11:00:00 GMT;";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/", creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/", creationDate));
   }
   {
     std::string str = "id=;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT(!c.getPersistent());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT(!c->getPersistent());
   }
   {
     std::string str = "id=; path=abc";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("/"), c.getPath());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("/"), c->getPath());
   }
   {
     std::string str = "id=; domain=.example.org";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "www.example.org", "/",creationDate));
+    CPPUNIT_ASSERT(cookie::parse(str, "www.example.org", "/",creationDate));
   }
   {
     // Fails because request host does not domain-match with cookie
     // domain.
     std::string str = "id=; domain=www.example.org";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "example.org", "/", creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "example.org", "/", creationDate));
   }
   {
     std::string str = "id=; domain=.";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/",creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/",creationDate));
   }
   {
     std::string str = "";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/",creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/",creationDate));
   }
   {
     std::string str = "=";
-    Cookie c;
-    CPPUNIT_ASSERT(!cookie::parse(c, str, "localhost", "/",creationDate));
+    CPPUNIT_ASSERT(!cookie::parse(str, "localhost", "/",creationDate));
   }
   {
     // Use domain last time seen.
     std::string str = "id=;domain=a.example.org;domain=.example.org";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "b.example.org", "/",creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("example.org"), c.getDomain());
+    auto c = cookie::parse(str, "b.example.org", "/",creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("example.org"), c->getDomain());
   }
   {
     // numeric host
     std::string str = "id=;";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "192.168.0.1", "/",creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), c.getDomain());
-    CPPUNIT_ASSERT(c.getHostOnly());
+    auto c = cookie::parse(str, "192.168.0.1", "/",creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), c->getDomain());
+    CPPUNIT_ASSERT(c->getHostOnly());
   }
   {
     // numeric host
     std::string str = "id=; domain=192.168.0.1";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "192.168.0.1", "/",creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), c.getDomain());
-    CPPUNIT_ASSERT(c.getHostOnly());
+    auto c = cookie::parse(str, "192.168.0.1", "/",creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), c->getDomain());
+    CPPUNIT_ASSERT(c->getHostOnly());
   }
   {
     // DQUOTE around cookie-value
     std::string str = "id=\"foo\";";
-    Cookie c;
-    CPPUNIT_ASSERT(cookie::parse(c, str, "localhost", "/", creationDate));
-    CPPUNIT_ASSERT_EQUAL(std::string("foo"), c.getValue());
+    auto c = cookie::parse(str, "localhost", "/", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("foo"), c->getValue());
+  }
+  {
+    // Default path
+    std::string str = "id=;";
+    auto c = cookie::parse(str, "localhost", "/foo", creationDate);
+    CPPUNIT_ASSERT(c);
+    CPPUNIT_ASSERT_EQUAL(std::string("/foo"), c->getPath());
   }
 }
 
